# ticket: t248
# mode: compile

from ewing8 cimport (Foo,


                     Blarg)

from declandimpl cimport (Sandwich


                          ,




                          Tomato)

cdef extern Foo yummy
