# mode: compile

from __future__ import nested_scopes

from __future__ import with_statement

pass

from __future__ import nested_scopes ; from __future__ import nested_scopes

from __future__ import print_function
print(end='')
