# mode: compile

"Welcome to the parrot module. It is currently resting."

def zap(polly, volts):
    "Wake up polly."

class Parrot:
    "Standard Norwegian Blue."

    def admire_plumage(self):
        "Lovely, ain't it?"

cdef class SuperParrot:
    "Special high-performance model."

